'use strict';

var _classCallCheck = require('babel-runtime/helpers/class-call-check').default;

var _regeneratorRuntime = require('babel-runtime/regenerator').default;

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default').default;

exports.__esModule = true;

var _pinkie = require('pinkie');

var _pinkie2 = _interopRequireDefault(_pinkie);

var _testcafeBrowserNatives = require('testcafe-browser-natives');

var _testcafeLegacyApi = require('testcafe-legacy-api');

var _testcafeHammerhead = require('testcafe-hammerhead');

var _browserConnection = require('../browser-connection');

var _browserConnection2 = _interopRequireDefault(_browserConnection);

var _browserConnectionLocal = require('../browser-connection/local');

var _browserConnectionLocal2 = _interopRequireDefault(_browserConnectionLocal);

var _messages = require('../messages');

var _browserSet = require('./browser-set');

var _browserSet2 = _interopRequireDefault(_browserSet);

var Bootstrapper = (function () {
    function Bootstrapper(browserConnectionGateway) {
        _classCallCheck(this, Bootstrapper);

        this.browserConnectionGateway = browserConnectionGateway;

        this.sources = [];
        this.browsers = [];
        this.filter = null;
        this.reporter = null;
    }

    Bootstrapper._convertAliasOrPathToBrowserInfo = function _convertAliasOrPathToBrowserInfo(browser) {
        var browserInfo;
        return _regeneratorRuntime.async(function _convertAliasOrPathToBrowserInfo$(context$2$0) {
            while (1) switch (context$2$0.prev = context$2$0.next) {
                case 0:
                    if (!(typeof browser === 'string')) {
                        context$2$0.next = 7;
                        break;
                    }

                    context$2$0.next = 3;
                    return _regeneratorRuntime.awrap(_testcafeBrowserNatives.getBrowserInfo(browser));

                case 3:
                    browserInfo = context$2$0.sent;

                    if (browserInfo) {
                        context$2$0.next = 6;
                        break;
                    }

                    throw new Error(_messages.getText(_messages.MESSAGE.cantFindBrowser, browser));

                case 6:
                    return context$2$0.abrupt('return', browserInfo);

                case 7:
                    return context$2$0.abrupt('return', browser);

                case 8:
                case 'end':
                    return context$2$0.stop();
            }
        }, null, this);
    };

    Bootstrapper.prototype._createConnectionFromBrowserInfo = function _createConnectionFromBrowserInfo(browserInfo) {
        if (browserInfo instanceof _browserConnection2.default) return browserInfo;

        return new _browserConnectionLocal2.default(this.browserConnectionGateway, browserInfo);
    };

    Bootstrapper.prototype._getBrowserConnections = function _getBrowserConnections() {
        var browsers, browserConnections;
        return _regeneratorRuntime.async(function _getBrowserConnections$(context$2$0) {
            var _this = this;

            while (1) switch (context$2$0.prev = context$2$0.next) {
                case 0:
                    if (this.browsers.length) {
                        context$2$0.next = 2;
                        break;
                    }

                    throw new Error(_messages.getText(_messages.MESSAGE.browserNotSet));

                case 2:
                    context$2$0.next = 4;
                    return _regeneratorRuntime.awrap(_pinkie2.default.all(this.browsers.map(Bootstrapper._convertAliasOrPathToBrowserInfo)));

                case 4:
                    browsers = context$2$0.sent;
                    browserConnections = browsers.map(function (browser) {
                        return _this._createConnectionFromBrowserInfo(browser);
                    });
                    context$2$0.next = 8;
                    return _regeneratorRuntime.awrap(_browserSet2.default.from(browserConnections));

                case 8:
                    return context$2$0.abrupt('return', context$2$0.sent);

                case 9:
                case 'end':
                    return context$2$0.stop();
            }
        }, null, this);
    };

    Bootstrapper.prototype._getTests = function _getTests() {
        var compiler, tests;
        return _regeneratorRuntime.async(function _getTests$(context$2$0) {
            var _this2 = this;

            while (1) switch (context$2$0.prev = context$2$0.next) {
                case 0:
                    if (this.sources.length) {
                        context$2$0.next = 2;
                        break;
                    }

                    throw new Error(_messages.getText(_messages.MESSAGE.testSourcesNotSet));

                case 2:
                    compiler = new _testcafeLegacyApi.Compiler(this.sources, _testcafeHammerhead.wrapDomAccessors);
                    context$2$0.next = 5;
                    return _regeneratorRuntime.awrap(compiler.getTests());

                case 5:
                    tests = context$2$0.sent;

                    if (this.filter) tests = tests.filter(function (test) {
                        return _this2.filter(test.name, test.fixture.name, test.fixture.path);
                    });

                    if (tests.length) {
                        context$2$0.next = 9;
                        break;
                    }

                    throw new Error(_messages.getText(_messages.MESSAGE.noTestsToRun));

                case 9:
                    return context$2$0.abrupt('return', tests);

                case 10:
                case 'end':
                    return context$2$0.stop();
            }
        }, null, this);
    };

    Bootstrapper.prototype._getReporterPlugin = function _getReporterPlugin() {
        var pluginFactory = this.reporter;

        if (typeof pluginFactory !== 'function') {
            try {
                var alias = this.reporter || 'spec';

                pluginFactory = require('testcafe-reporter-' + alias);
            } catch (err) {
                throw new Error(_messages.getText(_messages.MESSAGE.cantFindReporterForAlias, this.reporter));
            }
        }

        return pluginFactory();
    };

    // API

    Bootstrapper.prototype.createRunnableConfiguration = function createRunnableConfiguration() {
        var reporterPlugin, tests, browserSet;
        return _regeneratorRuntime.async(function createRunnableConfiguration$(context$2$0) {
            while (1) switch (context$2$0.prev = context$2$0.next) {
                case 0:
                    reporterPlugin = this._getReporterPlugin();
                    context$2$0.next = 3;
                    return _regeneratorRuntime.awrap(this._getTests());

                case 3:
                    tests = context$2$0.sent;
                    context$2$0.next = 6;
                    return _regeneratorRuntime.awrap(this._getBrowserConnections());

                case 6:
                    browserSet = context$2$0.sent;
                    return context$2$0.abrupt('return', { reporterPlugin: reporterPlugin, browserSet: browserSet, tests: tests });

                case 8:
                case 'end':
                    return context$2$0.stop();
            }
        }, null, this);
    };

    return Bootstrapper;
})();

exports.default = Bootstrapper;
module.exports = exports.default;