'use strict';

var _inherits = require('babel-runtime/helpers/inherits').default;

var _classCallCheck = require('babel-runtime/helpers/class-call-check').default;

var _regeneratorRuntime = require('babel-runtime/regenerator').default;

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default').default;

exports.__esModule = true;

var _events = require('events');

var _pinkie = require('pinkie');

var _pinkie2 = _interopRequireDefault(_pinkie);

var _timeLimitPromise = require('time-limit-promise');

var _timeLimitPromise2 = _interopRequireDefault(_timeLimitPromise);

var _promisifyEvent = require('promisify-event');

var _promisifyEvent2 = _interopRequireDefault(_promisifyEvent);

var _noopFn = require('noop-fn');

var _noopFn2 = _interopRequireDefault(_noopFn);

var _browserConnectionLocal = require('../browser-connection/local');

var _browserConnectionLocal2 = _interopRequireDefault(_browserConnectionLocal);

var _messages = require('../messages');

var _utilsArrayRemove = require('../utils/array-remove');

var _utilsArrayRemove2 = _interopRequireDefault(_utilsArrayRemove);

var BrowserSet = (function (_EventEmitter) {
    _inherits(BrowserSet, _EventEmitter);

    function BrowserSet(connections) {
        var _this = this;

        _classCallCheck(this, BrowserSet);

        _EventEmitter.call(this);

        this.READY_TIMEOUT = 30000;
        this.RELEASE_TIMEOUT = 10000;

        this.pendingReleases = [];

        this.connections = connections;

        this.browserErrorHandler = function (error) {
            return _this.emit('error', error);
        };

        connections.forEach(function (bc) {
            return bc.on('error', _this.browserErrorHandler);
        });

        //NOTE: We're setting an empty error handler, because Node kills the process on an 'error' event
        //if there is no handler. See: https://nodejs.org/api/events.html#events_class_events_eventemitter
        this.on('error', _noopFn2.default);
    }

    BrowserSet._waitIdle = function _waitIdle(bc) {
        return _regeneratorRuntime.async(function _waitIdle$(context$2$0) {
            while (1) switch (context$2$0.prev = context$2$0.next) {
                case 0:
                    if (!(bc.idle || bc.closed || !bc.ready)) {
                        context$2$0.next = 2;
                        break;
                    }

                    return context$2$0.abrupt('return');

                case 2:
                    context$2$0.next = 4;
                    return _regeneratorRuntime.awrap(_pinkie2.default.race([_promisifyEvent2.default(bc, 'idle'), _promisifyEvent2.default(bc, 'closed')]));

                case 4:
                case 'end':
                    return context$2$0.stop();
            }
        }, null, this);
    };

    BrowserSet._closeConnection = function _closeConnection(bc) {
        return _regeneratorRuntime.async(function _closeConnection$(context$2$0) {
            while (1) switch (context$2$0.prev = context$2$0.next) {
                case 0:
                    if (!(bc.closed || !bc.ready)) {
                        context$2$0.next = 2;
                        break;
                    }

                    return context$2$0.abrupt('return');

                case 2:

                    bc.close();

                    context$2$0.next = 5;
                    return _regeneratorRuntime.awrap(_promisifyEvent2.default(bc, 'closed'));

                case 5:
                case 'end':
                    return context$2$0.stop();
            }
        }, null, this);
    };

    BrowserSet.prototype._waitConnectionsReady = function _waitConnectionsReady() {
        var connectionsReadyPromise = _pinkie2.default.all(this.connections.filter(function (bc) {
            return !bc.ready;
        }).map(function (bc) {
            return _promisifyEvent2.default(bc, 'ready');
        }));

        var timeoutError = new Error(_messages.getText(_messages.MESSAGE.cantEstablishBrowserConnection));

        return _timeLimitPromise2.default(connectionsReadyPromise, this.READY_TIMEOUT, { rejectWith: timeoutError });
    };

    BrowserSet.prototype._checkForDisconnections = function _checkForDisconnections() {
        var disconnectedUserAgents = this.connections.filter(function (bc) {
            return bc.closed;
        }).map(function (bc) {
            return bc.userAgent;
        });

        if (disconnectedUserAgents.length) throw new Error(_messages.getText(_messages.MESSAGE.cantRunAgainstDisconnectedBrowsers, disconnectedUserAgents.join(', ')));
    };

    //API

    BrowserSet.from = function from(connections) {
        var _this2 = this;

        var browserSet = new BrowserSet(connections);

        var prepareConnection = _pinkie2.default.resolve().then(function () {
            browserSet._checkForDisconnections();
            return browserSet._waitConnectionsReady();
        }).then(function () {
            return browserSet;
        });

        return _pinkie2.default.race([prepareConnection, _promisifyEvent2.default(browserSet, 'error')]).catch(function callee$2$0(error) {
            return _regeneratorRuntime.async(function callee$2$0$(context$3$0) {
                while (1) switch (context$3$0.prev = context$3$0.next) {
                    case 0:
                        context$3$0.next = 2;
                        return _regeneratorRuntime.awrap(browserSet.dispose());

                    case 2:
                        throw error;

                    case 3:
                    case 'end':
                        return context$3$0.stop();
                }
            }, null, _this2);
        });
    };

    BrowserSet.prototype.releaseConnection = function releaseConnection(bc) {
        var _this3 = this;

        if (this.connections.indexOf(bc) < 0) return _pinkie2.default.resolve();

        _utilsArrayRemove2.default(this.connections, bc);

        bc.removeListener('error', this.browserErrorHandler);

        var appropriateStateSwitch = bc instanceof _browserConnectionLocal2.default ? BrowserSet._closeConnection(bc) : BrowserSet._waitIdle(bc);

        var release = _timeLimitPromise2.default(appropriateStateSwitch, this.RELEASE_TIMEOUT).then(function () {
            return _utilsArrayRemove2.default(_this3.pendingReleases, release);
        });

        this.pendingReleases.push(release);

        return release;
    };

    BrowserSet.prototype.dispose = function dispose() {
        return _regeneratorRuntime.async(function dispose$(context$2$0) {
            var _this4 = this;

            while (1) switch (context$2$0.prev = context$2$0.next) {
                case 0:
                    // FIXME: make a shallow copy of `this.connections`, because it
                    // will be modified inside `releaseConnection`.
                    // Not the most clear piece of code here. I hope we can do
                    // better. But I'm out of ideas for now.
                    this.connections.slice().forEach(function (bc) {
                        return _this4.releaseConnection(bc);
                    });

                    context$2$0.next = 3;
                    return _regeneratorRuntime.awrap(_pinkie2.default.all(this.pendingReleases));

                case 3:
                case 'end':
                    return context$2$0.stop();
            }
        }, null, this);
    };

    return BrowserSet;
})(_events.EventEmitter);

exports.default = BrowserSet;
module.exports = exports.default;