'use strict';

var _inherits = require('babel-runtime/helpers/inherits').default;

var _classCallCheck = require('babel-runtime/helpers/class-call-check').default;

var _regeneratorRuntime = require('babel-runtime/regenerator').default;

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default').default;

exports.__esModule = true;

var _pinkie = require('pinkie');

var _pinkie2 = _interopRequireDefault(_pinkie);

var _promisifyEvent = require('promisify-event');

var _promisifyEvent2 = _interopRequireDefault(_promisifyEvent);

var _path = require('path');

var _events = require('events');

var _flatten = require('flatten');

var _flatten2 = _interopRequireDefault(_flatten);

var _bootstrapper = require('./bootstrapper');

var _bootstrapper2 = _interopRequireDefault(_bootstrapper);

var _reporter2 = require('../reporter');

var _reporter3 = _interopRequireDefault(_reporter2);

var _task = require('./task');

var _task2 = _interopRequireDefault(_task);

var Runner = (function (_EventEmitter) {
    _inherits(Runner, _EventEmitter);

    function Runner(proxy, browserConnectionGateway) {
        _classCallCheck(this, Runner);

        _EventEmitter.call(this);

        this.proxy = proxy;
        this.bootstrapper = new _bootstrapper2.default(browserConnectionGateway);

        this.opts = {
            screenshotPath: null,
            takeScreenshotsOnFails: false,
            skipJsErrors: false,
            quarantineMode: false,
            reportOutStream: void 0
        };
    }

    // Run task

    Runner.prototype._runTask = function _runTask(reporterPlugin, browserSet, tests) {
        var task, reporter;
        return _regeneratorRuntime.async(function _runTask$(context$2$0) {
            while (1) switch (context$2$0.prev = context$2$0.next) {
                case 0:
                    task = new _task2.default(tests, browserSet.connections, this.proxy, this.opts);
                    reporter = new _reporter3.default(reporterPlugin, task, this.opts.reportOutStream);

                    task.on('browser-job-done', function (job) {
                        return browserSet.releaseConnection(job.browserConnection);
                    });

                    context$2$0.prev = 3;
                    context$2$0.next = 6;
                    return _regeneratorRuntime.awrap(_pinkie2.default.race([_promisifyEvent2.default(task, 'done'), _promisifyEvent2.default(browserSet, 'error')]));

                case 6:
                    context$2$0.next = 15;
                    break;

                case 8:
                    context$2$0.prev = 8;
                    context$2$0.t0 = context$2$0['catch'](3);

                    task.abort();
                    task.removeAllListeners();

                    context$2$0.next = 14;
                    return _regeneratorRuntime.awrap(browserSet.dispose());

                case 14:
                    throw context$2$0.t0;

                case 15:
                    context$2$0.next = 17;
                    return _regeneratorRuntime.awrap(browserSet.dispose());

                case 17:
                    return context$2$0.abrupt('return', reporter.testCount - reporter.passed);

                case 18:
                case 'end':
                    return context$2$0.stop();
            }
        }, null, this, [[3, 8]]);
    };

    // API

    Runner.prototype.src = function src() {
        for (var _len = arguments.length, sources = Array(_len), _key = 0; _key < _len; _key++) {
            sources[_key] = arguments[_key];
        }

        sources = _flatten2.default(sources).map(function (path) {
            return _path.resolve(path);
        });

        this.bootstrapper.sources = this.bootstrapper.sources.concat(sources);

        return this;
    };

    Runner.prototype.browsers = function browsers() {
        for (var _len2 = arguments.length, _browsers = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
            _browsers[_key2] = arguments[_key2];
        }

        this.bootstrapper.browsers = this.bootstrapper.browsers.concat(_flatten2.default(_browsers));

        return this;
    };

    Runner.prototype.reporter = function reporter(_reporter, outStream) {
        this.bootstrapper.reporter = _reporter;
        this.opts.reportOutStream = outStream;

        return this;
    };

    Runner.prototype.filter = function filter(_filter) {
        this.bootstrapper.filter = _filter;

        return this;
    };

    Runner.prototype.screenshots = function screenshots(path) {
        var takeOnFails = arguments.length <= 1 || arguments[1] === undefined ? false : arguments[1];

        this.opts.takeScreenshotsOnFails = takeOnFails;
        this.opts.screenshotPath = path;

        return this;
    };

    Runner.prototype.run = function run() {
        var _ref,
            skipJsErrors,
            quarantineMode,
            _ref2,
            reporterPlugin,
            browserSet,
            tests,
            args$2$0 = arguments;

        return _regeneratorRuntime.async(function run$(context$2$0) {
            while (1) switch (context$2$0.prev = context$2$0.next) {
                case 0:
                    _ref = args$2$0.length <= 0 || args$2$0[0] === undefined ? {} : args$2$0[0];
                    skipJsErrors = _ref.skipJsErrors;
                    quarantineMode = _ref.quarantineMode;

                    this.opts.skipJsErrors = !!skipJsErrors;
                    this.opts.quarantineMode = !!quarantineMode;

                    context$2$0.next = 7;
                    return _regeneratorRuntime.awrap(this.bootstrapper.createRunnableConfiguration());

                case 7:
                    _ref2 = context$2$0.sent;
                    reporterPlugin = _ref2.reporterPlugin;
                    browserSet = _ref2.browserSet;
                    tests = _ref2.tests;

                    this.emit('done-bootstrapping');

                    context$2$0.next = 14;
                    return _regeneratorRuntime.awrap(this._runTask(reporterPlugin, browserSet, tests));

                case 14:
                    return context$2$0.abrupt('return', context$2$0.sent);

                case 15:
                case 'end':
                    return context$2$0.stop();
            }
        }, null, this);
    };

    return Runner;
})(_events.EventEmitter);

exports.default = Runner;
module.exports = exports.default;